@@ -1,9 +1,6 @@
 package Foo;
 
 use Function::Interface::Impl qw(IFoo);
-
-use Function::Return;
-use Function::Parameters;
 use Types::Standard -types;
 
 fun hello(Str $msg) :Return(Str) {
