@@ -1,7 +1,5 @@
 package Bar;
 use Function::Interface::Impl qw(IBar);
-use Function::Parameters;
-use Function::Return;
 
 fun bar() :Return() { }
 
