@@ -1,7 +1,5 @@
 package FooBar;
 use Function::Interface::Impl qw(IFoo IBar);
-use Function::Parameters;
-use Function::Return;
 
 fun foo() :Return() { }
 fun bar() :Return() { }
